@@ -33,19 +33,13 @@ sub arguments {
     my $self = shift;
     my $arguments = $self->SUPER::arguments(@_);
 
-    for my $column ( $self->record->columns ) {
-        if ( not $column->writable and $column->readable ) {
-            $arguments->{$column->name}{'render_mode'} = 'read';
-        }
-    }
-
     for my $pk (@{ $self->record->_primary_keys }) {
         $arguments->{$pk}{'constructor'} = 1;
-        $arguments->{$pk}{'mandatory'} = 1;
         # XXX TODO IS THERE A BETTER WAY TO NOT RENDER AN ITEM IN arguments
         $arguments->{$pk}{'render_as'} = 'Unrendered'; 
         # primary key fields should always be hidden fields
     }
+    $arguments->{delete} = {render_as => "Unrendered"};
     return $arguments;
 }
 
@@ -57,17 +51,13 @@ going to change the record.  This is opposed to the behavior inherited
 from L<Jifty::Action>, where mandatory arguments B<must> be present
 for the action to run.
 
-However, constructor arguments are still required.
-
 =cut
 
 sub _validate_arguments {
-    my $self = shift;
-
-    $self->_validate_argument($_) for grep {
-        $self->has_argument($_)
-            or $self->arguments->{$_}->{constructor}
-    } $self->argument_names;
+    my $self   = shift;
+    
+    $self->_validate_argument($_)
+      for grep {exists $self->argument_values->{$_}} $self->argument_names;
 
     return $self->result->success;
 }
@@ -84,62 +74,54 @@ sub take_action {
     my $self = shift;
     my $changed = 0;
 
-    my $event_info = $self->_setup_event_before_action();
-
     for my $field ( $self->argument_names ) {
+
         # Skip values that weren't submitted
-        next unless $self->has_argument($field);
+        next unless exists $self->argument_values->{$field};
 
         my $column = $self->record->column($field);
 
         # Skip nonexistent fields
         next unless $column;
 
-        # Grab the value
-        my $value = $self->argument_value($field);
-
         # Boolean and integer fields should be skipped if blank.
         # (This logic should be moved into SB or something.)
         next
             if ( defined $column->type and ( $column->type =~ /^bool/i || $column->type =~ /^int/i )
-            and defined $value and $value eq '' );
-
-        # Skip file uploads if blank
-        next if lc $self->arguments->{$field}{render_as} eq "upload"
-          and (not defined $value or not ref $value);
-
-        if (ref $value eq "Fh") { # CGI.pm's "lightweight filehandle class"
-            local $/;
-            binmode $value;
-            $value = scalar <$value>;
-        }
+            and defined $self->argument_value($field) and $self->argument_value($field) eq '' );
 
         # Skip fields that have not changed
         my $old = $self->record->$field;
-        # XXX TODO: This ignore "by" on columns
-        $old = $old->id if ref($old) and $old->isa( 'Jifty::Record' );
+        $old = $old->id if UNIVERSAL::isa( $old, "Jifty::Record" );
     
         # if both the new and old values are defined and equal, we don't want to change em
-        # XXX TODO "$old" is a cheap hack to scalarize datetime objects
-        next if ( defined $old and defined $value and "$old" eq "$value" );
+        next if ( defined $old and defined $self->argument_value($field) and $old eq $self->argument_value($field) );
 
+        
         # If _both_ the values are ''
         next if (  (not defined $old or not length $old)
-                    and ( not defined $value or not length $value ));
+                    and ( not defined $self->argument_value($field) or not length $self->argument_value($field) ));
 
         my $setter = "set_$field";
-        my ( $val, $msg ) = $self->record->$setter( $value );
+        my ( $val, $msg ) = $self->record->$setter( $self->argument_value($field) );
         $self->result->field_error($field, $msg)
           if not $val and $msg;
 
         $changed = 1 if $val;
     }
 
+    # XXX: This should be only on ::Delete 
+    if ($self->argument_value("delete")) {
+        my ( $val, $msg ) = $self->record->delete;
+        $self->result->error($msg)
+          if not $val and $msg;
+
+        $changed = 1 if $val;
+    }
+
     $self->report_success
       if $changed and not $self->result->failure;
 
-    $self->_setup_event_after_action($event_info);
-
     return 1;
 }
 
@@ -153,7 +135,7 @@ user-friendly result.
 
 sub report_success {
     my $self = shift;
-    $self->result->message(_("Updated"))
+    $self->result->message("Updated")
 }
 
 1;
