@@ -2,11 +2,11 @@ use warnings;
 use strict;
 
 package Jifty::LetMe;
-use Digest::MD5 ();
-use Math::BigInt::Calc;
-use String::Koremutake ();
+use Digest::MD5;
+use Math::BigInt;
+use String::Koremutake;
 
-use base qw/Jifty::Object Class::Accessor::Fast/;
+use base qw/Jifty::Object Class::Accessor/;
 
 __PACKAGE__->mk_accessors ( qw/checksum_provided email path args until user/);
 
@@ -19,14 +19,6 @@ Jifty::LetMe - A way to expose single-link URLs to your applications
 Create a new "LetMe" authentication object; it takes no parameters.
 It calls L</_init> to do any initialization.
 
-A LetMe is a way to provide a one-time-use URL for a particular purpose.
-All LetMe objects give you a way to validate someone's identity and to
-allow them a very small set of possible actions or page-access permissions.
-
-For example, you can put a LetMe URL in an email to a new user,
-so that when they click on the URL you know that their email address
-is valid.
-
 =cut
 
 sub new {
@@ -80,11 +72,25 @@ for the user who has the email address I<ADDRESS>.
 sub _user_from_email {
     my $self = shift;
     my $email = shift;
-    my $currentuser_object_class = Jifty->app_class("CurrentUser");
+    my $currentuser_object_class = Jifty->config->framework('CurrentUserClass');
+    $currentuser_object_class->require;
     return $currentuser_object_class->new( email => $email );
+
 }
 
-sub _generate_digest {
+
+=head2 generate_checksum
+
+Returns an auth checksum for the current combination of
+
+    user
+    token
+    until
+
+=cut
+
+
+sub generate_checksum {
     my $self = shift;
 
     # get user's generic secret
@@ -96,57 +102,17 @@ sub _generate_digest {
     $digest->add( $user->auth_token );
     $digest->add( $self->path );
     my %args = %{$self->args};
-    $digest->add( Encode::encode_utf8($_), Encode::encode_utf8($args{$_})) for sort keys %args;
+    $digest->add( $_, $args{$_}) for sort keys %args;
     $digest->add( $self->until ) if ($self->until);
-    return $digest->hexdigest();
-}
+    # only take the first 16 characters. we're rally just trying to 
+    # get something reasonably short, memorable and unguessable
+    my $integer_digest = Math::BigInt->new("0x".substr($digest->hexdigest(),0,16));
 
+    
+    # the scary version:
+    #my $integer_digest = Math::BigInt->new("0x". $digest->hexdigest() );
 
-
-=head2 generate_checksum
-
-Returns an auth checksum for the current combination of
-
-    user
-    path
-    arguments
-    until
-
-=cut
-
-sub generate_checksum {
-    my $self = shift;
-
-    return substr( $self->_generate_digest, 0, 16 );
-}
-
-=head2 generate_koremutake_checksum
-
-Generate a slightly more pronouncable version of the checksum using
-L<String::Koremutake>.  Due to hex -> integer limitations, this is
-imporecise and may vary depending on the platform it is used on; as
-such, it is deprecated.
-
-=cut
-
-sub generate_koremutake_checksum {
-    my $self = shift;
-
-    # Only take the first 16 characters. We're really just trying to
-    # get something reasonably short, memorable and unguessable. Also,
-    # don't use Math::BigInt->new directly for simple computation,
-    # because it insists exporting overload to us, which makes
-    # devel::cover and devel::dprof very sad.  This is deprecated in
-    # favor of generate_checksum, which returns a straight hex digest.
-    my $integer_digest = Math::BigInt::Calc->_str(
-        Math::BigInt::Calc->_from_hex(
-            substr( $self->_generate_digest, 0, 16 )
-        )
-    );
-
-    # koremutake it.  This loses precision, since most perls can't
-    # deal with 64 bits with precision.  Thus, $integer_digest ends up
-    # being rounded, possibly in unpredicatable ways.
+    # koremutake it
     my $k = String::Koremutake->new;
     return( $k->integer_to_koremutake($integer_digest));
 
@@ -164,7 +130,7 @@ into
       token => 'update_task/23'
       until => 20050101,
       checksum_provided => bekidrikufryvagygefuba
-
+ 
 =cut
 
 sub from_token {
@@ -173,13 +139,13 @@ sub from_token {
 
     my @atoms = split('/',$token);
 
-    $self->email( Jifty::I18N->maybe_decode_utf8(URI::Escape::uri_unescape( shift @atoms )) );
+    $self->email( URI::Escape::uri_unescape( shift @atoms ) );
     $self->path( shift @atoms );
     $self->checksum_provided( pop @atoms );
 
-    my %args = map { Jifty::I18N->maybe_decode_utf8(URI::Escape::uri_unescape($_)) } @atoms;
+    my %args = @atoms;
     $self->until( delete $args{until} ) if $args{until};
-
+    
     $self->args(\%args);
 }
 
@@ -205,7 +171,7 @@ for passing in a URL
 
 sub as_encoded_token {
     my $self = shift;
-    $self->_generate_token( email => URI::Escape::uri_escape_utf8($self->email) );
+    $self->_generate_token( email => URI::Escape::uri_escape($self->email) );
 }
 
 sub _generate_token {
@@ -214,7 +180,7 @@ sub _generate_token {
     return join ('/', 
         $args{'email'},
         $self->path,
-        (map {URI::Escape::uri_escape_utf8($_)} %{$self->args}),
+        %{$self->args},
         (defined $self->until ? ( 'until', $self->until ) : () ), #?
         $self->generate_checksum  
         );
@@ -231,7 +197,7 @@ Jifty->web->url, L</base_path> and L</as_encoded_token>
 
 sub as_url {
     my $self = shift;
-    return Jifty->web->url(path => $self->base_path . $self->as_encoded_token);
+    return Jifty->web->url . $self->base_path. $self->as_encoded_token;
 
 }
 
@@ -241,7 +207,7 @@ sub as_url {
 By default, all "LetMe" actions live at URLs under '/let' inside your
 application.  Override this subroutine to change that.
 
-By default, it returns '/let/'
+By default, it returns '/let'
 
 =cut
 
@@ -292,10 +258,7 @@ actually do much input checking. You want to call "validate"
 
 sub _correct_checksum_provided {
     my $self = shift;
-    return undef
-        unless ( $self->checksum_provided eq $self->generate_checksum )
-        or
-        ( $self->checksum_provided eq $self->generate_koremutake_checksum );
+    return undef unless ($self->checksum_provided eq $self->generate_checksum); 
 
 }
 
