@@ -1,10 +1,10 @@
-=begin properties
+=for properties
 
 constructor
 canonicalizer
 available_values
 ajax_validates
-autocompleter
+ajax_autocomplete
 
 default_value
 valid_values
@@ -12,11 +12,9 @@ validator
 render_as
 label
 hints
-placeholder
 length
 mandatory
 
-=end properties
 
 =cut
 
@@ -42,14 +40,11 @@ aid in placing them in L<HTML::Mason> components.
 
 =cut
 
-use base 'Jifty::Web::Form::Element';
+use base qw/Jifty::Web::Form::Element Class::Accessor/;
 
 use Scalar::Util;
 use HTML::Entities;
-
-# We need the anonymous sub because otherwise the method of the base class is
-# always called, instead of the appropriate overridden method in a child class.
-use overload '""' => sub { shift->render }, bool => sub { 1 };
+use overload '""' => sub {shift->render};
 
 =head2 new
 
@@ -60,33 +55,35 @@ Should only be called from C<< $action->arguments >>.
 
 sub new {
     my $class = shift;
-    my $self = $class->SUPER::new(
-      { type          => 'text',
+    my $self = bless {}, $class;
+
+    my %args = (
+        type          => 'text',
         class         => '',
         input_name    => '',
         default_value => '',
         sticky_value  => '',
-        render_mode   => 'update' });
-    my $args = ref($_[0]) ? $_[0] : {@_};
+        render_mode   => 'update',
+        @_,
+    );
 
-    my $subclass = ucfirst($args->{render_as} || $args->{type} || 'text');
-    $subclass = 'Jifty::Web::Form::Field::' . $subclass unless $subclass =~ /::/;
-    bless $self, $subclass if Jifty::Util->require($subclass);
+    my $subclass;
+    if ($args{render_as}) {
+        $subclass = ucfirst($args{render_as});
+    } elsif ($args{'type'}) {
+        $subclass = ucfirst($args{'type'});
+    }
+    if ($subclass) { 
+        $subclass = 'Jifty::Web::Form::Field::' . $subclass unless $subclass =~ /::/;
+        if ( $subclass->require() ) {
+            bless $self, $subclass;
+        }
+    }
 
     for my $field ( $self->accessors() ) {
-        $self->$field( $args->{$field} ) if exists $args->{$field};
+        $self->$field( $args{$field} ) if exists $args{$field};
     }
 
-    # If they key and/or value imply that this argument is going to be
-    # a mapped argument, then do the mapping and mark the field as hidden.
-    my ($key, $value) = Jifty::Request::Mapper->query_parameters($self->input_name, $self->current_value);
-    if ($key ne $self->input_name) {
-        require Jifty::Web::Form::Field::Hidden;
-        bless $self, "Jifty::Web::Form::Field::Hidden";
-        $self->input_name($key);
-        $self->default_value($value);
-        $self->sticky_value(undef);
-    }
 
     # now that the form field has been instantiated, register the action with the form.
     if ($self->action and not (Jifty->web->form->has_action($self->action))) {
@@ -104,8 +101,8 @@ C<new>.  Subclasses should extend this list.
 
 =cut
 
-sub accessors { shift->SUPER::accessors(), qw(name label input_name type sticky sticky_value default_value action mandatory ajax_validates ajax_canonicalizes autocompleter preamble hints placeholder focus render_mode length _element_id); }
-__PACKAGE__->mk_accessors(qw(name _label _input_name type sticky sticky_value default_value _action mandatory ajax_validates ajax_canonicalizes autocompleter preamble hints placeholder focus render_mode length _element_id));
+sub accessors { shift->SUPER::accessors(), qw(name label input_name type sticky sticky_value default_value action mandatory ajax_validates ajax_autocomplete preamble hints render_mode length _element_id); }
+__PACKAGE__->mk_accessors(qw(name _label _input_name type sticky sticky_value default_value _action mandatory ajax_validates ajax_autocomplete preamble hints render_mode length _element_id));
 
 =head2 name [VALUE]
 
@@ -129,6 +126,7 @@ Gets or sets the type of the HTML <input> field -- that is, 'text' or
 
 Sets this form field's "submit" key binding to VALUE. 
 
+
 =head2 default_value [VALUE]
 
 Gets or sets the default value for the form.
@@ -137,32 +135,7 @@ Gets or sets the default value for the form.
 
 Gets or sets the value for the form field that was submitted in the last action.
 
-=head2 mandatory [VALUE]
-
-A boolean indicating that the argument B<must> be present when the
-user submits the form.
-
-=head2 focus [VALUE]
-
-If true, put focus on this form field when the page loads.
-
-=head2 ajax_validates [VALUE]
-
-A boolean value indicating if user input into an HTML form field for
-this argument should be L<validated|Jifty::Manual::Glossary/validate>
-via L<AJAX|Jifty::Manual::Glossary/AJAX> as the user fills out the
-form, instead of waiting until submit. Arguments will B<always> be
-validated before the action is run, whether or not they also
-C<ajax_validate>.
-
-=head2 ajax_canonicalizes [VALUE]
-
-A boolean value indicating if user input into an HTML form field for
-this argument should be L<canonicalized|Jifty::Manual::Glossary/canonicalize>
-via L<AJAX|Jifty::Manual::Glassary/AJAX> as the user fills out the
-form, instead of waiting until submit.  Arguments will B<always> be
-canonicalized before the action is run, whether or not they also
-C<ajax_canonicalize>
+=cut
 
 =head2 id 
 
@@ -191,12 +164,12 @@ sub input_name {
 # Otherwise, we should ask our action, how to turn our "name"
 # into a form input name.
 
-    my $ret = $self->_input_name(@_);
-    return $ret if $ret;
-
-    my $action = $self->action;
-    return $action ? $self->action->form_field_name( $self->name )
-                   : '';
+    $self->_input_name(@_)
+      || (
+          $self->action
+        ? $self->action->form_field_name( $self->name )
+        : ''
+      );
 }
 
 
@@ -297,10 +270,9 @@ sub current_value {
 =head2 render
 
 Outputs this form element in a span with class C<form_field>.  This
-outputs the label, the widget itself, any hints, any errors, and any
-warnings using L</render_label>, L</render_widget>, L</render_hints>,
-L</render_errors>, and L</render_warnings>, respectively.  Returns an
-empty string.
+outputs the label, the widget itself, any hints, and any errors, using
+L</render_label>, L</render_widget>, L</render_hints>,
+L</render_errors> respectively.  Returns an empty string.
 
 This is also what C<Jifty::Web::Form::Field>s do when stringified.
 
@@ -315,12 +287,10 @@ sub render {
     $self->render_label();
     if ($self->render_mode eq 'update') { 
         $self->render_widget();
-        $self->render_autocomplete_div();
-        $self->render_inline_javascript();
+        $self->render_autocomplete();
+        $self->render_key_binding();
         $self->render_hints();
         $self->render_errors();
-        $self->render_warnings();
-        $self->render_canonicalization_notes();
     } elsif ($self->render_mode eq 'read'){ 
         $self->render_value();
     }
@@ -328,36 +298,6 @@ sub render {
     return ('');
 }
 
-=head2 render_inline_javascript
-
-Render a <script> tag (if neceesary) containing any inline javascript
-that should follow this form field. This is used to add an
-autocompleter, placeholder, or keybinding to form fields where needed.
-
-=cut
-
-sub render_inline_javascript {
-    my $self = shift;
-
-    my $javascript;
-
-    $javascript = join(
-        "\n",
-        grep {$_} (
-            $self->autocomplete_javascript(),
-            $self->placeholder_javascript(),
-            $self->key_binding_javascript(),
-            $self->focus_javascript()
-        )
-    );
-    
-    if($javascript =~ /\S/) {
-        Jifty->web->out(qq{<script type="text/javascript"><!--
-    $javascript
---></script>
-});
-    }
-}
 
 =head2 classes
 
@@ -368,8 +308,7 @@ Renders a default CSS class for each part of our widget.
 
 sub classes {
     my $self = shift;
-    my $name = $self->name;
-    return join(' ', ($self->class||''), ($name ? "argument-".$name : ''));
+    return join(' ', ($self->class||''), ($self->name||''));
 }
 
 
@@ -381,10 +320,7 @@ Output the start of div that wraps the form field
 
 sub render_wrapper_start {
     my $self = shift;
-    my @classes = qw(form_field);
-    if ($self->mandatory) { push @classes, 'mandatory' }
-    if ($self->name)      { push @classes, 'argument-'.$self->name }
-    Jifty->web->out('<div class="'.join(' ', @classes).'">' ."\n");
+    Jifty->web->out('<div class="form_field">' ."\n");
 }
 
 =head2 render_wrapper_end
@@ -412,7 +348,7 @@ Use this for sticking instructions right in front of a widget
 sub render_preamble {
     my $self = shift;
     Jifty->web->out(
-qq!<span class="preamble @{[$self->classes]}" >@{[_($self->preamble) || '' ]}</span>\n!
+qq!<span class="preamble @{[$self->classes]}" >@{[$self->preamble || '' ]}</span>\n!
     );
 
     return '';
@@ -428,15 +364,9 @@ an empty string.
 
 sub render_label {
     my $self = shift;
-    if ( $self->render_mode eq 'update' ) {
-        Jifty->web->out(
-qq!<label class="label @{[$self->classes]}" for="@{[$self->element_id ]}">@{[_($self->label) ]}</label>\n!
-        );
-    } else {
-        Jifty->web->out(
-            qq!<span class="label @{[$self->classes]}">@{[_($self->label) ]}</span>\n!
-        );
-    }
+    Jifty->web->out(
+qq!<label class="label @{[$self->classes]}" for="@{[$self->input_name ]}">@{[$self->label ]}</label>\n!
+    );
 
     return '';
 }
@@ -454,7 +384,7 @@ sub render_widget {
     my $self  = shift;
     my $field = qq!  <input !;
     $field .= qq! type="@{[ $self->type ]}"!;
-    $field .= qq! name="@{[ $self->input_name ]}"! if ($self->input_name);
+    $field .= qq! name="@{[ $self->input_name ]}"!;
     $field .= qq! id="@{[ $self->element_id ]}"!;
     $field .= qq! value="@{[HTML::Entities::encode_entities($self->current_value)]}"! if defined $self->current_value;
     $field .= $self->_widget_class; 
@@ -467,15 +397,11 @@ sub render_widget {
 
 =head2 other_widget_properties
 
-If your widget subclass has other properties it wants to insert into the html
-of the main widget and you haven't subclassed render_widget then you can just
-subclass this.
+If your widget subclass has other properties it wants to insert into the html of the main widget and you haven't subclassed render_widget,
 
-If you have subclassed render_widget then just stick them in your local sub
-render_widget.
+just stick them in your local sub render_widget.
 
-We use this for marking password fields as not-autocomplete so the browser does
-not try to use its form autocompletion on them.
+We use this for marking password fields as not-autocomplete
 
 
 =cut
@@ -490,13 +416,7 @@ Returns the "class=" line for our widget. Optionally takes extra classes to appe
 
 sub _widget_class {
     my $self = shift;
-    my @classes = ( 'widget',
-                    $self->classes,
-                    ( $self->ajax_validates     ? ' ajaxvalidation' : '' ),
-                    ( $self->ajax_canonicalizes ? ' ajaxcanonicalization' : '' ),
-                    ( $self->autocompleter      ? ' ajaxautocompletes' : '' ),
-                    ( $self->focus              ? ' focus' : ''),
-                    @_ );
+    my @classes = ('widget', $self->classes, ($self->ajax_validates ? ' ajaxvalidation' : ''),@_);
 
     return qq! class="!. join(' ',@classes).  qq!"!
 
@@ -513,8 +433,7 @@ sub render_value {
     my $self  = shift;
     my $field = '<span';
     $field .= qq! class="@{[ $self->classes ]}"> !;
-    # XXX: force stringify the value because maketext is buggy with overloaded objects.
-    $field .= HTML::Entities::encode_entities(_("@{[$self->current_value]}")) if defined $self->current_value;
+    $field .= HTML::Entities::encode_entities($self->current_value) if defined $self->current_value;
     $field .= qq!</span>\n!;
     Jifty->web->out($field);
     return '';
@@ -522,89 +441,27 @@ sub render_value {
 
 
 
-=head2 render_autocomplete_div
-
-Renders an empty div that /__jifty/autocomplete.xml can fill
-in. Returns an empty string.
-
-=cut
-
-sub render_autocomplete_div { 
-    my $self = shift;
-    return unless($self->autocompleter);
-    Jifty->web->out(
-qq!<div class="autocomplete" id="@{[$self->element_id]}-autocomplete" style="display: none;"></div>!);
-
-    return '';
-}
-
 =head2 render_autocomplete
 
-Renders the div tag and javascript necessary to do autocompletion for
-this form field. Deprecated internally in favor of
-L</render_autocomplete_div> and L</autocomplete_javascript>, but kept
-for backwards compatability since there exists external code that uses
-it.
+Renders an empty div that /jifty/autocomplete.xml can fill in. Also renders the tiny snippet
+of javascript to make that call if necessary.
+Returns an empty string.
 
 =cut
 
-sub render_autocomplete {
+sub render_autocomplete { 
     my $self = shift;
-    return unless $self->autocompleter;
-    $self->render_autocomplete_div;
-    Jifty->web->out(qq{<script type="text/javascript"><!--
-    @{[$self->autocomplete_javascript]}
---></script>});
+    return unless($self->ajax_autocomplete);
+    Jifty->web->out(
+qq!<div class="autocomplete" id="@{[$self->element_id]}-autocomplete" style="display:none;border:1px solid black;background-color:white;"></div>\n
+        <script type="text/javascript">
+          new Jifty.Autocompleter('@{[$self->element_id]}', '@{[$self->element_id]}-autocomplete', '/jifty/autocomplete.xml')
+        </script>
+  !
+    );
+
     return '';
-}
 
-
-
-=head2 autocomplete_javascript
-
-Returns renders the tiny snippet of javascript to make an autocomplete
-call, if necessary.
-
-=cut
-
-sub autocomplete_javascript {
-    my $self = shift;
-    return unless($self->autocompleter);
-    return qq{new Jifty.Autocompleter('@{[$self->element_id]}','@{[$self->element_id]}-autocomplete')};
-}
-
-=head2 placeholder_javascript
-
-Returns the javascript necessary to insert a placeholder into this
-form field (greyed-out text that is written in using javascript, and
-vanishes when the user focuses the field). 
-
-=cut
-
-sub placeholder_javascript {
-    my $self = shift;
-    return unless $self->placeholder;
-    my $placeholder = $self->placeholder;
-    $placeholder =~ s{(['\\])}{\\$1}g;
-    $placeholder =~ s{\n}{\\n}g;
-    $placeholder =~ s{\r}{\\r}g;
-    return qq{new Jifty.Placeholder('@{[$self->element_id]}', '$placeholder');};
-}
-
-=head2 focus_javascript
-
-Returns the javascript necessary to focus this form field on page
-load, if necessary.
-
-=cut
-
-sub focus_javascript {
-    my $self = shift;
-    return undef;
-    if($self->focus) {
-        return qq{document.getElementById("@{[$self->element_id]}").focus()};
-        return qq{DOM.Events.addListener( window, "load", function(){document.getElementById("@{[$self->element_id]}").focus()})};
-    }
 }
 
 =head2 render_hints
@@ -617,7 +474,7 @@ subclasses commonly override this.  Returns an empty string.
 sub render_hints { 
     my $self = shift;
     Jifty->web->out(
-qq!<span class="hints @{[$self->classes]}">@{[_($self->hints) || '']}</span>\n!
+qq!<span class="hints @{[$self->classes]}">@{[$self->hints || '']}</span>\n!
     );
 
     return '';
@@ -638,43 +495,9 @@ sub render_errors {
     return unless $self->action;
 
     Jifty->web->out(
-qq!<span class="error @{[$self->classes]}" id="@{[$self->action->error_div_id($self->name)]}">@{[  $self->action->result->field_error( $self->name ) || '']}</span>\n!
-    );
-    return '';
-}
-
-=head2 render_warnings
-
-Outputs a <div> with any warnings for this action, even if there are
-none -- AJAX could fill it in.
-
-=cut
-
-sub render_warnings {
-    my $self = shift;
-
-    return unless $self->action;
-
-    Jifty->web->out(
-qq!<span class="warning @{[$self->classes]}" id="@{[$self->action->warning_div_id($self->name)]}">@{[  $self->action->result->field_warning( $self->name ) || '']}</span>\n!
-    );
-    return '';
-}
-
-=head2 render_canonicalization_notes
-
-Outputs a <div> with any canonicalization notes for this action, even if there are
-none -- AJAX could fill it in.
-
-=cut
-
-sub render_canonicalization_notes {
-    my $self = shift;
-
-    return unless $self->action;
-
-    Jifty->web->out(
-qq!<span class="canonicalization_note @{[$self->classes]}" id="@{[$self->action->canonicalization_note_div_id($self->name)]}">@{[$self->action->result->field_canonicalization_note( $self->name ) || '']}</span>\n!
+qq!<span class="error @{[$self->classes]}" id="@{[$self->action->error_div_id($self->name)]}">
+      @{[  $self->action->result->field_error( $self->name ) || '']}
+    </span>\n!
     );
     return '';
 }
